-- ---------------------------------------------------------------------
-- Filename: tb.vhd 
-- Version: 1.0 
--
-- Date: April 2000
--
-- Type: Configuration Specification
-- 
-- Description: Testbench to stimulate the design top.vhd
--		    Note that there is a configuration in a
--		    seperate VHDL file for each macro, for 
--		    simulation only.
--		    
-- ---------------------------------------------------------------------
-- XAPP 409                                 
-- Simulating a 3.1i Core generator VHDL Design
--                                           
-- Instantiate a Core Generator BRAM configured as 32x16 and a 
-- bi-directional bus for the DATA in/out. The design example 
-- used will have a CORE Generator macro down two levels in the 
-- hierarchy to illustrate how parameters are passed to the macros 
-- for functional simulation. The RAMB4_S16_S16 component is a 
-- 4096-bit dual-ported dedicated random access memory blocks with 
-- synchronous write capability. Each port is independent of the other 
-- while accessing the same set of 4096 memory cells. This example will 
-- not exercise the entire depth of the block RAM.
--                                                       
-- Limited Warranty and Disclaimer. These designs are provided to you 
-- as is. Xilinx and its licensors make and you receive no warranties 
-- or conditions, express, implied, statutory or otherwise, and Xilinx 
-- specifically disclaims any implied warranties of merchantability, 
-- non-infringement, or fitness for a particular purpose. Xilinx does 
-- not warrant that the functions contained in these designs will meet 
-- your requirements, or that the operation of these designs will be 
-- uninterrupted or error free, or that defects in the Designs will be 
-- corrected. Furthermore, Xilinx does not warrant or make any 
-- representations regarding use or the results of the use of the 
-- designs in terms of correctness, accuracy, reliability, or otherwise. 
-- 
-- Limitation of Liability. In no event will Xilinx or its licensors be 
-- liable for any loss of data, lost profits, cost or procurement of 
-- substitute goods or services, or for any special, incidental, 
-- consequential, or indirect damages arising from the use or operation 
-- of the designs or accompanying documentation, however caused and on 
-- any theory of liability. This limitation will apply even if Xilinx 
-- has been advised of the possibility of such damage. This limitation 
-- shall apply not-withstanding the failure of the essential purpose of 
-- any limited remedies herein. 
--
-- ---------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use std.textio.all;
use std.standard.all;

-- use work.conv_utils.all;

entity ex_blkram_tb is
end;

architecture tb of ex_blkram_tb is

  -- --------------------------------------
  -- Declare the top level design component 
  -- --------------------------------------
  component top
    port(CLK         : in std_logic;
         WEA, WEB    : in std_logic;
         out_enable  : in std_logic;
         ADDRA, ADDRB: in std_logic_vector(4 downto 0);
         DATA_BUSA, DATA_BUSB : inout std_logic_vector(15 downto 0));
  end component;

  -- -----------------------------------
  -- Declare the signal in the testbench
  -- -----------------------------------
  SIGNAL CLK_TB, WEA_TB, WEB_TB, TRST_TB, OUT_EN_TB : std_logic;
  SIGNAL DATA_BUSA_TB, DATA_BUSB_TB : std_logic_vector(15 downto 0);
  SIGNAL ADDRA_TB, ADDRB_TB : std_logic_vector(4 downto 0);

  --SIGNAL expectedA : std_logic_vector(15 downto 0) := X"AAAA";
  --SIGNAL expectedB : std_logic_vector(15 downto 0) := X"5555";
  --SIGNAL parx : std_logic_vector(15 downto 0);

  CONSTANT CLK_PERIOD : time := 20 ns;

BEGIN

    -- ----------
    -- Define CLK
    -- ----------
    procCLK : process 		-- Create procCLK process
    begin
      CLK_TB <= '0';		-- Initialize CLK_TB 
      wait for 102 ns;          -- Wait for Reset to finish
       LOOP 			-- Create the CLK
         Wait for (CLK_PERIOD/2);
         CLK_TB <= not (CLK_TB);
        END LOOP;
    end process;

    -- ------------
    -- Define ADDR
    -- ------------
    procADDR : process(CLK_TB, TRST_TB)
    begin
	if (TRST_TB = '1') then
	    ADDRA_TB <= "00000";	
	    ADDRB_TB <= "11111";
	elsif (CLK_TB = '1' and CLK_TB'event) then
	    ADDRA_TB <= ADDRA_TB + "00001" after 10 ns;	-- increment address A
	    ADDRB_TB <= ADDRB_TB - "00001" after 10 ns;	-- decrement address B
	end if;
    end process;

    -- ------------------------	 
    -- Define test reset signal
    -- ------------------------	 
    TRST_TB <= '1',		-- testbench signal to reset address counters
	       '0' after 100 ns;

    -- ----------------------------
    -- Define BlockRAM write enable 
    -- ----------------------------
      WEA_TB <= '0',
	        '1' after 745 ns,		-- enable write
		'0' after 1378 ns;		-- Disable write mode
      WEB_TB <= '0';				-- Disable write mode

    -- ----------------------------------------------------
    -- Apply stimulus to the DATA_BUSA_TB bidir port (INPUT path)
    -- ----------------------------------------------------
 
   DATA_BUSA_TB <= (others => 'Z') after 50 ns,  -- No driver on input path
                   X"FFFF" after 742 ns,	-- force data to input path
                   X"0000" after 760 ns,	-- to write to the BRAM
                   X"FF00" after 780 ns,
                   X"00FF" after 800 ns,
                   X"F0F0" after 820 ns,
                   X"0F0F" after 840 ns,
                   X"BBFF" after 860 ns,
                   X"FFBB" after 880 ns,
                   X"B0B0" after 900 ns,
                   X"0B0B" after 920 ns,
                   X"1111" after 940 ns,
                   (others => 'Z') after 1380 ns;

    -- ----------------------------------------------------
    -- Apply stimulus to the DATA_BUSB_TB bidir port (INPUT path)
    -- ----------------------------------------------------
 
   DATA_BUSB_TB <= (others => 'Z') after 0 ns;  -- No driver on input path

    -- --------------------------------------
    -- Enable the output enable (OUTPUT path)
    -- --------------------------------------
      OUT_EN_TB <= '0' after 100 ns,   -- disable output from BRAM to bidir	
                   '1' after 200 ns, 	-- enable output from BRAM to bidir
                   '0' after 732 ns,
                   '1' after 1385 ns; 

    -- ----------------------
    -- Instantiate the design
    -- ----------------------

    uut : top port map (
	CLK => CLK_TB,
	WEA => WEA_TB,
	WEB => WEB_TB,
	out_enable => OUT_EN_TB,
        ADDRA => ADDRA_TB,
        ADDRB => ADDRB_TB,
	DATA_BUSA => DATA_BUSA_TB,
	DATA_BUSB => DATA_BUSB_TB
	);

END tb;


